@@ -1,8 +1,8 @@
-/*  $Id: BoolExpr.cpp,v 1.24 2010/02/14 19:28:30 sarrazip Exp $
+/*  $Id: BoolExpr.cpp,v 1.29 2013/03/03 08:19:27 sarrazip Exp $
     BoolExpr.cpp - Boolean expression binary tree node
 
     boolstuff - Disjunctive Normal Form boolean expression library
-    Copyright (C) 2002-2010 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2002-2013 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #ifndef _H_BoolExpr
@@ -59,6 +59,8 @@ BoolExpr<T>::BoolExpr(Type t, BoolExpr *l, BoolExpr *r)
     std::cerr << "CONS " << (void *) this << " " << type << " " << value << std::endl;
     #endif
     assert(type == AND || type == OR || type == NOT);
+    assert((type == NOT) == (!l && r));  // only a right subtree for a negation
+    assert((type == AND || type == OR) == (l && r));  // two subtrees for a binary operator
 }
 
 
@@ -239,6 +241,38 @@ BoolExpr<T>::getDNFTermRoots(OutputIter dest) const
 }
 
 
+// This expression is not expected to contain OR nodes.
+//
+template <class T>
+template <class OutputIter>
+OutputIter
+BoolExpr<T>::getDNFFactorRoots(OutputIter dest) const
+{
+    switch (type)
+    {
+	case AND:
+	    assert(left != NULL);
+	    assert(right != NULL);
+	    dest = left->getDNFFactorRoots(dest);
+	    return right->getDNFFactorRoots(dest);
+
+	case OR:
+            return dest;
+
+	case NOT:
+	    assert(right != NULL);
+	    assert(left == NULL);
+	case VALUE:
+	    *dest++ = const_cast<BoolExpr<T> *>(this);
+	    return dest;
+
+	default:
+	    assert(false);
+	    return dest;
+    }
+}
+
+
 template <class T>
 void
 BoolExpr<T>::getTreeVariables(
@@ -421,7 +455,7 @@ static int exprComparator(BoolExpr<T> *a, BoolExpr<T> *b)
 // Returns true iff term a comes before term b.
 //
 template <class T>
-static bool termOrdering(BoolExpr<T> *a, BoolExpr<T> *b)
+static bool exprOrdering(BoolExpr<T> *a, BoolExpr<T> *b)
 {
     return exprComparator(a, b) < 0;
 }
@@ -432,25 +466,24 @@ template <class T>
 BoolExpr<T> *
 BoolExpr<T>::getDisjunctiveNormalForm(BoolExpr<T> *root)
 {
-    BoolExpr<T> *dnfRoot = getRawDNF(root);
+    bool tooLarge = false;
+    BoolExpr<T> *dnfRoot = getRawDNF(root, tooLarge);
     if (dnfRoot == NULL)
-	return dnfRoot;
+	return NULL;
 
     // Get the terms of the DNF:
     std::vector<BoolExpr<T> *> termRoots;
     dnfRoot->getDNFTermRoots(std::inserter(termRoots, termRoots.end()));
 
+    simplifyConjunctionList(termRoots);
+
     // If only one term, consider it "useful",
     // because we must return something:
     if (termRoots.size() <= 1)
-    {
-	// Simplify terms of the form x & x.
-	simplifyXAndXTerms(termRoots);
 	return termRoots.front();
-    }
 
     // Sort terms to find repetitions:
-    std::sort(termRoots.begin(), termRoots.end(), termOrdering<T>);
+    std::sort(termRoots.begin(), termRoots.end(), exprOrdering<T>);
 
     // Determine which terms are useful (terms that don't always evaluate to false,
     // and are not the same as a previous term):
@@ -460,18 +493,19 @@ BoolExpr<T>::getDisjunctiveNormalForm(BoolExpr<T> *root)
     typename std::vector<BoolExpr<T> *>::const_iterator previous = termRoots.end();
     for (it = termRoots.begin(); it != termRoots.end(); previous = it, it++)
     {
-	assert(*it != NULL);
-	bool u = (*it)->isDNFTermUseful();
+        BoolExpr<T> *t = *it;
+	assert(t != NULL);
+	bool u = t->isDNFTermUseful();
 
 	// If 'it' is not the first term in termRoots, and 'it' is equal to
 	// the previous term in termRoots, then 'it' is not useful.
 	//
-	if (previous != termRoots.end() && exprComparator(*previous, *it) == 0)
+	if (previous != termRoots.end() && exprComparator(*previous, t) == 0)
 	    u = false;
 
 	isUsefulTerm.push_back(u);
 	if (u)
-	    usefulTerms.push_back(const_cast<BoolExpr<T> *>(*it));
+	    usefulTerms.push_back(const_cast<BoolExpr<T> *>(t));
     }
     assert(isUsefulTerm.size() == termRoots.size());
 
@@ -479,7 +513,7 @@ BoolExpr<T>::getDisjunctiveNormalForm(BoolExpr<T> *root)
     if (usefulTerms.size() == termRoots.size())
 	return dnfRoot;
 
-    destroyDNFOrNodes(dnfRoot);
+    destroyDNFBinaryOpNodes(dnfRoot, true);
     // dnfRoot now invalid; termRoots holds the term trees
 
     // If all the terms are useless (e.g., a&!a | b&!b),
@@ -499,14 +533,80 @@ BoolExpr<T>::getDisjunctiveNormalForm(BoolExpr<T> *root)
 	if (!isUsefulTerm[it - termRoots.begin()])
 	    delete *it;
 
-    // Simplify terms of the form x & x.
-    simplifyXAndXTerms(usefulTerms);
-
     // Join the useful terms with OR nodes:
     return joinTreesWithOrNodes(usefulTerms);
 }
 
 
+template <class T>
+/*static*/
+void
+BoolExpr<T>::simplifyConjunctionList(std::vector<BoolExpr<T> *> &conjunctionList)
+{
+    typename std::vector<BoolExpr<T> *>::iterator jt;
+    for (jt = conjunctionList.begin(); jt != conjunctionList.end(); jt++)
+	*jt = simplifyConjunction(*jt);
+}
+
+
+template <class T>
+/*static*/
+BoolExpr<T> *
+BoolExpr<T>::simplifyConjunction(BoolExpr<T> *conj)
+{
+    if (conj == NULL)
+        return NULL;
+
+    // From a&!b&c, get a vector containing a, !b, c.
+    //
+    std::vector<BoolExpr<T> *> factorRoots;
+    conj->getDNFFactorRoots(std::inserter(factorRoots, factorRoots.end()));
+
+    // Sort the factors to find the duplicates.
+    //
+    std::sort(factorRoots.begin(), factorRoots.end(), exprOrdering<T>);
+
+    BoolExpr<T> *previous = NULL;
+    typename std::vector<BoolExpr<T> *>::iterator it;
+    for (it = factorRoots.begin(); it != factorRoots.end(); it++)
+    {
+        BoolExpr<T> *f = *it;
+        assert(f != NULL);
+        assert(f->type == VALUE || f->type == NOT);  // expecting factor of a DNF
+        assert(f->type != NOT || (!f->left && f->right && f->right->type == VALUE));
+
+        if (previous && exprComparator(previous, f) == 0)
+        {
+            // This factor is the same as the previous.
+            // Destroy it and remove it from factorRoots[].
+            delete f;
+            *it = NULL;
+        }
+        else
+            previous = f;
+    }
+
+    destroyDNFBinaryOpNodes(conj, false);
+    // conj now invalid; factorRoots holds the term trees
+
+    // Build new conjunction from non null pointers in factorRoots[].
+    BoolExpr<T> *newConj = NULL;
+    for (it = factorRoots.begin(); it != factorRoots.end(); it++)
+    {
+        BoolExpr<T> *f = *it;
+        if (f == NULL)
+            continue;
+
+        if (newConj == NULL)  // if first factor of new conjunction
+            newConj = f;
+        else
+            newConj = new BoolExpr<T>(AND, newConj, f);
+    }
+
+    return newConj;
+}
+
+
 template <class T>
 /*static*/
 void
@@ -529,22 +629,29 @@ BoolExpr<T>::simplifyXAndXTerms(std::vector<BoolExpr<T> *> &terms)
 }
 	    
 
+// destroyOrNodes: if true, OR nodes are destroyed; otherwise,
+//                 AND nodes are destroyed.
+//
 template <class T>
 /*static*/
 void
-BoolExpr<T>::destroyDNFOrNodes(BoolExpr<T> *root)
+BoolExpr<T>::destroyDNFBinaryOpNodes(BoolExpr<T> *root, bool destroyOrNodes)
 {
-    if (root == NULL || root->type != OR)
+    if (root == NULL)
 	return;
 
-    // Detach the OR node's subtrees, so that 'delete' does not affect them:
+    Type targetedType = (destroyOrNodes ? OR : AND);
+    if (root->type != targetedType)
+	return;
+
+    // Detach the targeted node's subtrees, so that 'delete' does not affect them:
     BoolExpr<T> *left = root->left, *right = root->right;
     assert(left != NULL && right != NULL);
     root->left = root->right = NULL;
     delete root;
 
-    destroyDNFOrNodes(left);
-    destroyDNFOrNodes(right);
+    destroyDNFBinaryOpNodes(left,  destroyOrNodes);
+    destroyDNFBinaryOpNodes(right, destroyOrNodes);
 }
 
 
@@ -560,9 +667,10 @@ BoolExpr<T>::joinTreesWithOrNodes(const std::vector<BoolExpr<T> *> &trees)
 
     typename std::vector<BoolExpr<T> *>::const_iterator it = trees.begin();
 
-    BoolExpr<T> *result = new BoolExpr<T>(OR, *it++, NULL);
-    result->right = *it++;
-	// *it++ not used twice in same statement because eval order is undef
+    BoolExpr<T> *left = *it++;
+    BoolExpr<T> *right = *it++;
+    assert(left && right);
+    BoolExpr<T> *result = new BoolExpr<T>(OR, left, right);
 
     for ( ; it != trees.end(); it++)
 	result = new BoolExpr<T>(OR, result, *it);
@@ -571,20 +679,48 @@ BoolExpr<T>::joinTreesWithOrNodes(const std::vector<BoolExpr<T> *> &trees)
 }
 
 
+#if 0  // debugging code
+template <class T>
+static void printTree(const char *prefix, BoolExpr<T> *tree, size_t level)
+{
+    std::cout << prefix << ": L" << level << ", tree@" << (void *) tree;
+    if (tree)
+        std::cout << ": " << tree->print();
+    std::cout << std::endl;
+}
+
+class Decrementer
+{
+public:
+    Decrementer(size_t &r) : counter(r) {}
+    ~Decrementer() { --counter; }
+    size_t &counter;
+};
+#endif
+
+
 template <class T>
 /*static*/
 BoolExpr<T> *
-BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
+BoolExpr<T>::getRawDNF(BoolExpr<T> *root, bool &tooLarge)
 {
+#if 0  // debugging code
+    static size_t level = 0;
+    ++level;
+    Decrementer d(level);
+#endif
+
+    tooLarge = false;
+
     if (root == NULL)
         return NULL;
-    
+
     /*  One-level trees:
     */
     if (root->type == VALUE)
         return root;
     
-    /*  Two levels trees:
+    /*  Two-level trees:
     */
     assert(root->right != NULL);
     switch (root->type)
@@ -609,10 +745,15 @@ BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
 
     /*  Three or more levels:
     */
-    root->left = getRawDNF(root->left);
+    root->left = getRawDNF(root->left, tooLarge);
+    if (tooLarge)
+        return NULL;
     assert(root->left == NULL || root->left->isDisjunctiveNormalForm());
-    root->right = getRawDNF(root->right);
-    assert(root->right->isDisjunctiveNormalForm());
+
+    root->right = getRawDNF(root->right, tooLarge);
+    if (tooLarge)
+        return NULL;
+    assert(root->right == NULL || root->right->isDisjunctiveNormalForm());
 
 
     /*  Here, because of some simplifications, we may now have
@@ -644,10 +785,7 @@ BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
 	BoolExpr<T> *x = root->right;
 	root->right = NULL;
 	delete root;
-	return negateDNF(x);
-
-        assert(false);
-        return NULL;
+	return negateDNF(x, tooLarge);  // returns null if resulting expression would be too large
     }
 
 
@@ -658,18 +796,18 @@ BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
     /*  If one side is a value, make sure that this value is at the left:
     */
     if (root->left->type != VALUE && root->right->type == VALUE)
-        swap(root->left, root->right);
+        std::swap(root->left, root->right);
 
 
     /*  Permutate the left and right subtrees if they are
 	not in our "conventional order":
     */
     if (root->left->type == NOT && root->right->type == OR)
-        swap(root->left, root->right);
+        std::swap(root->left, root->right);
     else if (root->left->type == NOT && root->right->type == AND)
-        swap(root->left, root->right);
+        std::swap(root->left, root->right);
     else if (root->left->type == OR && root->right->type == AND)
-        swap(root->left, root->right);
+        std::swap(root->left, root->right);
 
 
     /*  Conventional order:
@@ -744,7 +882,7 @@ BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
             orNode->left = andNode;
             orNode->right = newAndNode;
 
-            return getRawDNF(orNode);
+            return getRawDNF(orNode, tooLarge);
         }
 
         if (root->left->type == AND && root->right->type == AND)
@@ -794,7 +932,7 @@ BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
             andNode->right = notNode;
             notNode->right = c;
 
-            return getRawDNF(orNode);
+            return getRawDNF(orNode, tooLarge);
         }
 
         assert(root->right->type == OR);
@@ -831,7 +969,7 @@ BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
             rightOrNode->right = thirdNewAndNode;
 
             BoolExpr<T> *newRoot = new BoolExpr<T>(OR, leftOrNode, rightOrNode);
-            return getRawDNF(newRoot);
+            return getRawDNF(newRoot, tooLarge);
         }
 
         if (root->left->type == AND)
@@ -852,7 +990,7 @@ BoolExpr<T>::getRawDNF(BoolExpr<T> *root)
 
             rightOrNode->left = topAndNode;
             rightOrNode->right = newHighAndNode;
-            return getRawDNF(rightOrNode);
+            return getRawDNF(rightOrNode, tooLarge);
         }
 
         assert(false);
@@ -899,11 +1037,16 @@ public:
 };
 
 
+/*  tooLarge: indicates if the resulting expression would be too large;
+              check this when the returned pointer is null.
+*/
 template <class T>
 /*static*/
 BoolExpr<T> *
-BoolExpr<T>::negateDNF(BoolExpr<T> *root)
+BoolExpr<T>::negateDNF(BoolExpr<T> *root, bool &tooLarge)
 {
+    tooLarge = false;
+
     if (root == NULL)
 	return NULL;
 
@@ -940,7 +1083,8 @@ BoolExpr<T>::negateDNF(BoolExpr<T> *root)
 	    if (product > max / variables.size())
 	    {
 		delete root;
-		return NULL;  // resulting expression would be too large
+                tooLarge = true;  // resulting expression would be too large
+		return NULL;
 	    }
 
 	    product *= variables.size();
