@@ -1,4 +1,4 @@
-/*  $Id: checkc.c,v 1.14 2007/08/26 18:45:26 sarrazip Exp $
+/*  $Id: checkc.c,v 1.17 2013/03/03 06:48:45 sarrazip Exp $
 
     boolstuff - Disjunctive Normal Form boolean expression library
     Copyright (C) 2002-2005 Pierre Sarrazin <http://sarrazip.com/>
@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 /*  
@@ -34,8 +34,12 @@
 #define RIGHT(x) boolstuff_get_right_subtree(x)
 #define TYPE(x) boolstuff_get_node_type(x)
 
+#ifndef __GNUC__
+#define __attribute__(x)
+#endif
+
 int
-main(int argc, char *argv[])
+main(__attribute__((unused)) int argc, __attribute__((unused)) char * argv[])
 {
     boolexpr_t upper_and_node, lower_and_node, or_node, not_node,
     		a_node, b_node, c_node, d_node,
